/*--------------------------------------------------------------------------*/
/* FILE: xfermdm7.c    (MODEM7 file name transfer)                          */
/*                                                                          */
/*                                                                          */
/*               The Opus Computer-Based Conversation System                */
/*       (c) Copyright 1986, Wynn Wagner III, All Rights Reserved           */
/*                                                                          */
/*                                                                          */
/* (MSC/4 with /Zp /Ox)                                                     */
/*                                                                          */
/*                                                                          */
/*                                                                          */
/*                                                                          */
/*  This module is similar to a routine used by Opus-Cbcs (1.00).  It is    */
/*  provided for your information only.  You will find routines that need   */
/*  to be coded and identifiers to be resolved.                             */
/*                                                                          */
/*  There is absolutely no guarantee that anything here will work.  If you  */
/*  break this routine, you own both pieces.                                */
/*                                                                          */
/*  USAGE:  You may use this material in any program with no obligation     */
/*          as long as there is no charge for your program.  For more       */
/*          information about commercial use, contact the "OPUSinfo HERE"   */
/*          BBS (124/111).                                                  */
/*                                                                          */
/*                                                                          */
/*--------------------------------------------------------------------------*/

#include "xfer.h"
#include "com.h"
#include <ctype.h>


extern char    *_s_mdm7fname;
extern char    *_s_mdm7giveup;
extern char    *_s_mdm7idunno;
extern char    *_s_mdm7chksum;
extern char    *_s_mdm7nak;
extern char    *_s_mdm7toolong;

/*--------------------------------------------------------------------------*/
/* XFERMDM7                                                                 */
/*--------------------------------------------------------------------------*/
int 
xfermdm7 (fn)
unsigned char  *fn;
{
   unsigned char   checksum;
   int             i, j;
   unsigned char   mdm7_head[13];
   unsigned char   num_tries = '\0';
   char           *fname;
   struct FILEINFO dta;
   long            t1, timerset ();

   XON_DISABLE ();
   dfind (&dta, fn, 0);
   fname = dta.name;

   /*--------------------------------------------------------------------*/
   /* Build a Modem7 style head                                          */
   /*--------------------------------------------------------------------*/
   memset (mdm7_head, ' ', 12);
   for (i = j = 0; ((fname[i]) && (i < 12) && (j < 12)); i++)
      {
      if (fname[i] == '.')
         j = 8;
      else
         mdm7_head[j++] = (char) toupper (fname[i]);
      }

   checksum = SUB;
   for (i = 0; i < 11; i++)
      checksum += mdm7_head[i];

   /*--------------------------------------------------------------------*/
   /* Top of the Name-Transfer Loop                                      */
   /*--------------------------------------------------------------------*/
top:
   if (!CARRIER)
      return 0;
   else
   if (num_tries++ > 10)
      {
      cputs (_s_mdm7giveup);
      send_can ();
      return (0);
      }
   else
   if (num_tries)
      SENDBYTE ('u');

   /*--------------------------------------------------------------------*/
   /* Wait for a Nak                                                     */
   /*--------------------------------------------------------------------*/
   for (i = 0; i < 15; i++)
      {
      if (!CARRIER)
         return (0);
      j = TIMED_READ (4);
      switch (j)
         {
         case 'C':
            SENDBYTE (SOH);
            return 2;
         case NAK:
            i = 16;
            break;
         case CAN:
            return 0;
         }
      }

   /*--------------------------------------------------------------------*/
   /* Transmit the file name                                             */
   /*--------------------------------------------------------------------*/
   SENDBYTE (ACK);
   for (i = 0; i < 11; i++)
      {
      SENDBYTE (mdm7_head[i]);
      switch (j = TIMED_READ (10))
         {
         case ACK:
            break;
         case CAN:
            SENDBYTE (ACK);
            return (0);
         default:              /* cprintf(_s_mdm7idunno,j); */
            goto top;
         }                      /* switch */
      }

try_sub:
   /*--------------------------------------------------------------------*/
   /* Transmit SUB and get CheckSum                                      */
   /*--------------------------------------------------------------------*/
   SENDBYTE (SUB);
   if ((i = TIMED_READ (10)) != checksum)
      {
      cputs (_s_mdm7chksum);
      goto top;
      }

   SENDBYTE (ACK);
   return (1);
}

/* END OF FILE: xfermdm7.c */
